package view;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.WindowConstants;

import main.Controller;
import main.ResultContent;
import model.databaseModel.Ansprechpartner;
import model.databaseModel.Unternehmen;
import model.databaseModel.Zustaendigkeit;

/**
 * 
 * Die Hauptfenster-Klasse.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: $ UTC
 */
@SuppressWarnings("serial")
public class MainFrame extends AbstractFrame implements Observer {
	private JScrollPane viewScrollPane = null;
	public JButton ansprechpartnerButton = null;
	public JButton unternehmenButton = null;
	public JButton zustaendigkeitButton = null;
	private Container contentPane = null;
	private final AnsprechpartnerTable ansprechpartnerTable = new AnsprechpartnerTable();
	private final UnternehmenTable unternehmenTable = new UnternehmenTable();
	private final ZustaendigkeitTable zustaendigkeitTable = new ZustaendigkeitTable();

	/**
	 * Konstruktor.
	 */
	public MainFrame(final Controller controller) {
		super(controller, "Aufgabe 10 fr Software-Engineering");

		// Make sure we have nice windows decorations.
		JFrame.setDefaultLookAndFeelDecorated(true);
		addWindowListener(new securityClosing());

		setLayout(new BorderLayout());
		contentPane = getContentPane();
		contentPane.add(new MenuBar(this), BorderLayout.NORTH);

		final JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(0, 3));
		ansprechpartnerButton = new JButton("Ansprechpartner");
		unternehmenButton = new JButton("Unternehmen");
		zustaendigkeitButton = new JButton("Zuordnungen");

		ansprechpartnerButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				setScrollPane(ansprechpartnerTable);
				controller.selectAnsprechpartner();
			}
		});

		unternehmenButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				setScrollPane(unternehmenTable);
				controller.selectUnternehmen();
			}
		});

		zustaendigkeitButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				setScrollPane(zustaendigkeitTable);
				controller.selectZustaendigkeit();
			}
		});

		/**
		 * Buttons registrieren.
		 */
		buttonPanel.add(ansprechpartnerButton, BorderLayout.WEST);
		buttonPanel.add(unternehmenButton, BorderLayout.WEST);
		buttonPanel.add(zustaendigkeitButton, BorderLayout.WEST);
		contentPane.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * Setze aktuellen ScrollPane im ContentPane
	 * 
	 * @param jTable
	 *            JTable
	 */
	private void setScrollPane(JTable jTable) {
		if (viewScrollPane != null) {
			contentPane.remove(viewScrollPane);
		}		
		viewScrollPane = new JScrollPane(jTable);
		contentPane.add(viewScrollPane, BorderLayout.CENTER);
		((AbstractTable) jTable).clearTable();
	}

	/**
	 * Benachrichtung vom Observer-Pattern
	 */
	@Override
	public void update(Observable arg0, Object arg1) {
		/**
		 * Daten in View bernehmen
		 */
		if (arg1 != null) {
			if (((ResultContent) arg1).getListType() instanceof Ansprechpartner) {
				setScrollPane(ansprechpartnerTable);
				List<Object> data = ((ResultContent) arg1).getListContent();
				for (Object f : data) {
					ansprechpartnerTable.dtm.addRow(new Object[] {
							((Ansprechpartner) f).getName(),
							((Ansprechpartner) f).getEmail() });
				}
			}

			if (((ResultContent) arg1).getListType() instanceof Unternehmen) {
				setScrollPane(unternehmenTable);
				List<Object> data = ((ResultContent) arg1).getListContent();
				for (Object f : data) {
					unternehmenTable.dtm
							.addRow(new Object[] { ((Unternehmen) f).getName() });
				}
			}

			if (((ResultContent) arg1).getListType() instanceof Zustaendigkeit) {
				setScrollPane(zustaendigkeitTable);
				List<Object> data = ((ResultContent) arg1).getListContent();
				for (Object f : data) {
					zustaendigkeitTable.dtm
							.addRow(new Object[] {
									((Zustaendigkeit) f).getUnternehmen()
											.getName(),
									((Zustaendigkeit) f).getAnsprechpartner()
											.getName() });
				}
			}
		} else {
			/**
			 * wenn kein Objekt zurckgeliefert wurde alle Daten aus dem RAM lschen.
			 */
			ansprechpartnerTable.clearTable();
			unternehmenTable.clearTable();
			zustaendigkeitTable.clearTable();
		}

		// Aufruf der validate-Methode fr Neuzeichnen notwendig!
		validate();
		repaint();
	}

	/**
	 * 
	 * private innere Klasse, um Schlieen ber X-Button optional abzufangen und
	 * auf jeden Fall die Datenbank ber DataBaseHelper.shutdown() wieder
	 * herunterzufahren.
	 * 
	 * Den Shutdown-Prozess vor dem Schlieen automatisch aufzurufen (bei
	 * Benutzung des X-Buttons) funktionierte nicht wenn die MainFrame-Klasse
	 * direkt den WindowsListener implementiert - das windowClosing-Event dort
	 * wurde dann nicht aufgerufen obwohl es implementiert war.
	 * 
	 * So funktioniert es. Man beachte dass die Ausgabe "Datenbank-Shutdown" in
	 * jedem Fall erfolgt, egal wie die Anwendung geschlossen wird.
	 * 
	 * @author $Author: Thomas Kramer
	 * @version $Revision: $, $Date: $ UTC
	 */
	private class securityClosing extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			int option = JOptionPane.showOptionDialog(MainFrame.this,
					"Wollen Sie das Programm wirklich verlassen?",
					"Sicherheitsabfrage", JOptionPane.YES_NO_OPTION,
					JOptionPane.WARNING_MESSAGE, null, null, null);

			if (option == JOptionPane.YES_OPTION) {
				/*
				 * expliziter DB-Shutdown nicht mehr ntig, im Connection-String
				 * wurde festgelegt dass das DBMS automatisch runterfhrt sobald
				 * alle Verbindungen gelst wurden
				 */
				Controller.em.close();
				Controller.emf.close();
				System.exit(0);
			} else {
				// Abbruch
				setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			}
		}
	}
}