package view;

import java.awt.Dialog.ModalityType;
import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import main.Controller;
import model.databaseModel.Unternehmen;
import model.databaseModel.Zustaendigkeit;

/**
 * MenuBar-Klasse, die eine Titelleiste erstellt.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: 1.0, $Date: 24.11.2012
 */

@SuppressWarnings("serial")
public class MenuBar extends JMenuBar {
	/**
	 * bergeordnetes Formular zwischenspeichern
	 */
	private MainFrame mainFrame = null;

	/**
	 * Konstruktor.
	 */
	public MenuBar(MainFrame frame) {
		super();
		mainFrame = frame;

		add(new FileMenu(frame));
		add(new DataMenu(frame));
		add(new HelpMenu(frame));
	}

	/**
	 * Die Dateimen-Klasse.
	 */
	private class FileMenu extends JMenu {
		/**
		 * 
		 * Konstruktor.
		 * 
		 * @param frame
		 *            Frame
		 */
		public FileMenu(final JFrame frame) {
			// Aufrufen des Basisklassenkonstruktors.
			super("Datei");

			// Anwendung beenden
			add(new AbstractAction("Beenden") {
				@Override
				public void actionPerformed(ActionEvent e) {
					/*
					 * expliziter DB-Shutdown nicht mehr ntig, im
					 * Connection-String wurde festgelegt dass das DBMS
					 * automatisch runterfhrt sobald alle Verbindungen gelst
					 * wurden
					 */
					Controller.em.close();
					Controller.emf.close();
					System.exit(0);
				}
			});
		}
	}

	/**
	 * Die Datenanlegen-Klasse.
	 */
	private class DataMenu extends JMenu {
		/**
		 * 
		 * Konstruktor.
		 * 
		 * @param frame
		 *            Frame
		 */
		public DataMenu(final JFrame frame) {
			// Aufrufen des Basisklassenkonstruktors.
			super("Daten");

			/**
			 * Ansprechpartner anlegen
			 */
			add(new AbstractAction("Ansprechpartner anlegen") {
				@Override
				public void actionPerformed(ActionEvent e) {
					JPanel neuerAnsprechpartner = new JPanel();
					neuerAnsprechpartner.setLayout(new BoxLayout(
							neuerAnsprechpartner, BoxLayout.Y_AXIS));
					JLabel nameLabel = new JLabel(
							"Bitte geben Sie den Namen ein:");
					JLabel emailLabel = new JLabel(
							"Bitte geben Sie die Email Adresse ein:");
					JLabel unternehmenLabel = new JLabel(
							"Bitte whlen Sie das Unternehmen aus:");

					JTextField nameInputField = new JTextField();
					JTextField emailInputField = new JTextField();
					List<Unternehmen> unternehmenListe = Controller
							.getUnternehmen();
					JComboBox unternehmenComboBox = new JComboBox();
					for (Unternehmen u : unternehmenListe) {
						unternehmenComboBox.addItem(u.getName());
					}

					neuerAnsprechpartner.add(nameLabel);
					neuerAnsprechpartner.add(nameInputField);
					neuerAnsprechpartner.add(emailLabel);
					neuerAnsprechpartner.add(emailInputField);
					neuerAnsprechpartner.add(unternehmenLabel);
					neuerAnsprechpartner.add(unternehmenComboBox);

					if (JOptionPane.showConfirmDialog(null,
							neuerAnsprechpartner,
							"Neuen Ansprechpartner anlegen",
							JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.PLAIN_MESSAGE) == 0) {

						if (nameInputField.getText().trim().equals("")) {
							JOptionPane.showMessageDialog(null,
									"Es wurden nicht alle Pflichtfelder ausgefllt.",
									"Fehler", JOptionPane.ERROR_MESSAGE);
						} else {
							if (mainFrame.controller
									.insertAnsprechpartnerAndZustaendigkeit(
											nameInputField.getText(),
											emailInputField.getText(),
											unternehmenListe.get(unternehmenComboBox
													.getSelectedIndex()))) {

								/*
								 * Erfolgsmeldung ausgeben
								 */
								JOptionPane
										.showMessageDialog(
												null,
												"Es wurde ein neuer Ansprechpartner mitsamt Unternehmenszuordnung angelegt!",
												"Information",
												JOptionPane.INFORMATION_MESSAGE);

							} else {
								/*
								 * Fehlermeldung ausgeben
								 */
								JOptionPane
										.showMessageDialog(
												null,
												"Beim Speichern ist ein Fehler aufgetreten!",
												"Fehler",
												JOptionPane.ERROR_MESSAGE);
							}
						}
					}
				}
			});

			/**
			 * Unternehmen anlegen
			 */
			add(new AbstractAction("Unternehmen anlegen") {
				@Override
				public void actionPerformed(ActionEvent e) {
					JPanel neuesUnternehmen = new JPanel();
					neuesUnternehmen.setLayout(new BoxLayout(neuesUnternehmen,
							BoxLayout.Y_AXIS));
					JLabel nameLabel = new JLabel(
							"Bitte geben Sie den Namen ein:");

					JTextField nameInputField = new JTextField();

					neuesUnternehmen.add(nameLabel);
					neuesUnternehmen.add(nameInputField);

					if (JOptionPane.showConfirmDialog(null, neuesUnternehmen,
							"Neues Unternehmen anlegen",
							JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.PLAIN_MESSAGE) == 0) {

						if (nameInputField.getText().trim().equals("")) {
							JOptionPane.showMessageDialog(null,
									"Es wurden nicht alle Pflichtfelder ausgefllt.",
									"Fehler", JOptionPane.ERROR_MESSAGE);
						} else {

							if (mainFrame.controller
									.insertUnternehmen(nameInputField.getText())) {

								/*
								 * Erfolgsmeldung ausgeben
								 */
								JOptionPane
										.showMessageDialog(
												null,
												"Es wurde ein neues Unternehmen angelegt!",
												"Information",
												JOptionPane.INFORMATION_MESSAGE);

							} else {
								/*
								 * Fehlermeldung ausgeben
								 */
								JOptionPane
										.showMessageDialog(
												null,
												"Beim Speichern ist ein Fehler aufgetreten!",
												"Fehler",
												JOptionPane.ERROR_MESSAGE);
							}
						}
					}
				}
			});

			/**
			 * Filter fr Ansprechpartner-Unternehmenszuordnungen anlegen
			 */
			add(new AbstractAction("Filter fr Zuordnungen festlegen") {
				@Override
				public void actionPerformed(ActionEvent e) {
					JPanel auswahlUnternehmen = new JPanel();
					auswahlUnternehmen.setLayout(new BoxLayout(
							auswahlUnternehmen, BoxLayout.Y_AXIS));
					JLabel unternehmenAuswahlLabel = new JLabel(
							"Bitte whlen Sie das Unternehmen aus:");
					JComboBox unternehmenComboBox = new JComboBox();

					List<Unternehmen> unternehmenListe = Controller
							.getUnternehmen();
					for (Unternehmen u : unternehmenListe) {
						unternehmenComboBox.addItem(u.getName());
					}

					auswahlUnternehmen.add(unternehmenAuswahlLabel);
					auswahlUnternehmen.add(unternehmenComboBox);

					if (JOptionPane
							.showConfirmDialog(
									null,
									auswahlUnternehmen,
									"Ansprechparter fr ein bestimmtes Unternehmen anzeigen",
									JOptionPane.OK_CANCEL_OPTION,
									JOptionPane.PLAIN_MESSAGE) == 0) {

						/**
						 * Filter ausfhren
						 */
						mainFrame.controller
								.selectZustaendigkeit(unternehmenListe
										.get(unternehmenComboBox
												.getSelectedIndex()));
					}
				}
			});

		}
	}

	/**
	 * Die Hilfsmen-Klasse
	 */
	private class HelpMenu extends JMenu {
		/**
		 * 
		 * Konstruktor.
		 * 
		 * @param frame
		 */
		public HelpMenu(final JFrame frame) {
			// Beschriftung
			super("Hilfe");

			// ffnen eines Infofensters.
			add(new AbstractAction("Info") {
				@Override
				public void actionPerformed(ActionEvent e) {
					JOptionPane.showMessageDialog(frame,
							"Eine Demo-Applikation fr HSQLDB.", "Info",
							JOptionPane.INFORMATION_MESSAGE);
				}
			});
		}
	}
}